/****************************************************************************
*   Function:   ChkDrv()
*   Version:    1.00
*
*   Author:     Ly Minh Tr
*   Date:       05/11/94
*   Copyright:  Copyright 1994, Tri M. Ly
*
*   Purpose:    A FoxPro API function to check for drive validity, floppy or
*               hard disk (will work for net work drives, too).  Code can
*               be compiled for both FoxPro for DOS and Windows.
*
*   Parameter:  Drive letter/number to be tested (0=default,A=1,B=2,C=3,etc.)
*
*   Returns:    .T. if drive is accessible, .F. otherwise.
*****************************************************************************/

#if defined(_WINDLL)
    #include <windows.h>
#endif
#include <dos.h>
#include <pro_ext.h>

//
// Pseudo-variables to access parameter and define library version #
//
#define pCount      parm->pCount
#define theVal      parm->p[0].val
#define theDrv      ((char FAR *) _HandToPtr(theVal.ev_handle))
#define parm1Hndl   (theVal.ev_handle)
#define parm1Len    (theVal.ev_length)

#define K_VERSION   "ChkDrv() Version 1.0"
#define K_INT24     0x24

//
// Function prototypes
//
int FAR _chkdrv(ParamBlk FAR *);
int FAR _chkver(void);

#if !defined(_WINDLL)
    static void (__interrupt FAR *_old_int24)();

    //
    // Define error code psuedo-constants
    //
    #define ERR_UNKNOWN     0x01
    #define ERR_NOT_READY   0x02
    #define ERR_INVALID_CMD 0x03
    #define ERR_GENERAL     0x0C

    /*
    *
    *   _new_int24()
    *
    *   Handle errors when attemping to access floppy drive with invalid
    *   media.
    *
    */
    static void __interrupt __far _new_int24
        (unsigned ES, unsigned DS, unsigned DI, unsigned SI,
         unsigned BP, unsigned SP, unsigned BX, unsigned DX,
         unsigned CX, unsigned AX)
    {
        switch (DI)
        {
            //
            // register DI contains the error code so we only need
            // to look at this register
            //
            case ERR_UNKNOWN:
            case ERR_NOT_READY:
            case ERR_INVALID_CMD:
            case ERR_GENERAL:
                //
                // reset register AX so DOS will ignore the error
                //
                AX = 0;
                break;
            default:
                //
                // if errors are not ours to worry about, then let old
                // error handler take care of it
                //
                (* _old_int24)();
        }
        return;
	}
#endif

//
//  Declare FoxInfo and FoxTable data structures
//
FoxInfo myFoxInfo[] = {
  { "CHKDRV", (FPFI) _chkdrv, 1, "?" },
  { "CHKVER", (FPFI) _chkver, 0, "" }
};

FoxTable _FoxTable = {
    (FoxTable FAR *) 0, sizeof(myFoxInfo)/sizeof(FoxInfo), myFoxInfo
};

/*
*
*   _chkdrv()
*
*   Checks validity of floppy and/or hard drive.
*
*/
int FAR _chkdrv(ParamBlk FAR *parm)
{
    if ((pCount != 1) || ((theVal.ev_type == 'C') && !isalpha(*theDrv)))
    {
        _Execute("WAIT WINDOW [ChkDrv() Error: Invalid parameter!] NOWAIT");
        _RetLogical(0);
    }
    else
    {
        int diskErr;
        struct _diskfree_t drive;

        #if defined(_WINDLL)
            //
            // Tells Windows not to display Critical-Error message...
            //
            SetErrorMode(SEM_FAILCRITICALERRORS);
        #else
            //
            // Save old critical error interrupt vector address and
            // set new address to our error handler routine
            //
            _old_int24 = _dos_getvect(K_INT24);
            _disable();
            _dos_setvect(K_INT24, _new_int24);
            _enable();
        #endif

        //
        // lock memory and fetch information about the given drive
        //
        switch (theVal.ev_type)
        {
            case 'C':
                _HLock(parm1Hndl);
                diskErr = _dos_getdiskfree(toupper(*theDrv)-'A'+1, &drive);
                _HUnLock(parm1Hndl);
                break;
            case 'I':
                diskErr = _dos_getdiskfree((int) theVal.ev_long, &drive);
                break;
            case 'N':
                diskErr = _dos_getdiskfree((int) theVal.ev_real, &drive);
                break;
            default:
                _Execute("WAIT WINDOW [ChkDrv() Error: "
                         "Invalid parameter!] NOWAIT");
                diskErr = 1;
        }

        #if defined(_WINDLL)
            //
            // Tells Windows to display Critical-Error message...
            //
            SetErrorMode(0);
        #else
            //
            // Restore old error handler routine
            //
            _disable();
            _dos_setvect(K_INT24, _old_int24);
            _enable();
        #endif

        //
        // Return result to FoxPro
        //
        _RetLogical(diskErr == 0);
    }

    return (0);
}

/*
*
*   _chkver()
*
*   Returns version number of library.
*
*/
int FAR _chkver(void)
{
    _RetChar(K_VERSION);
    return (0);
}
